###############################################################################
###############################################################################
##
##  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
##
##  This copyrighted material is made available to anyone wishing to use,
##  modify, copy, or redistribute it subject to the terms and conditions
##  of the GNU General Public License v.2.
##
###############################################################################
###############################################################################
%define _unpackaged_files_terminate_build 0
%define __os_install_post %{nil}

%define my_requires %{_builddir}/my_requires
%define _use_internal_dependency_generator 0

%define KERNEL_VERSION 2
%define KERNEL_PATCHLEVEL 4
%define KERNEL_SUBLEVEL 21
%define KERNEL_EXTRAVERSION 52.EL
%define kernel_version %{KERNEL_VERSION}.%{KERNEL_PATCHLEVEL}.%{KERNEL_SUBLEVEL}-%{KERNEL_EXTRAVERSION}

%define buildup 1
%define buildsmp 1
%define buildhugemem 1

%ifnarch i686
%define buildhugemem 0
%endif

%ifnarch i686 x86_64 athlon
%define buildsmp 0
%endif 

Name: GFS
Version: 6.0.2.36
Release: 5
License: GPL
GROUP: System Environment/Base
Summary: The GFS cluster file system and related products.
Obsoletes: GFS-gnbd GNBD
BuildRequires: rsync flex bison
BuildRequires: kernel-source = %{kernel_version}
ExclusiveArch: i386 i686 ia64 x86_64 ia32e
Requires: initscripts >= 7.31.18.EL-1
Conflicts: perl-Net-Telnet-3.03-3

Source0: gfs-%{version}.tar.gz
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-buildroot
%description
This package contains the Global File System (GFS) and related packages.

%package devel
Summary: GFS Libraries and header files
GROUP: System Environment/Base

%description devel
Libraries and header files.

%if %{buildup}
%package modules
GROUP: System Environment/Base
Summary: The GFS cluster file system and related products.
Requires: %{name} >= %{version}-%{release}
Requires: kernel = %{kernel_version}
Requires: /lib/modules/%{kernel_version}
Provides: kernel-modules

%description modules
This package contains the Global File System (GFS) kernel modules for a uniprocessor kernel.
%endif

%if %{buildsmp}
%package modules-smp
GROUP: System Environment/Base
Summary: The GFS cluster file system and related products.
Requires: %{name} >= %{version}-%{release}
Requires: kernel-smp = %{kernel_version}
Requires: /lib/modules/%{kernel_version}smp
Provides: kernel-modules

%description modules-smp
This package contains the Global File System (GFS) kernel modules for an SMP kernel.
%endif

%if %{buildhugemem}
%package modules-hugemem
GROUP: System Environment/Base
Summary: The GFS cluster file system and related products.
Requires: %{name} >= %{version}-%{release}
Requires: kernel-hugemem = %{kernel_version}
Requires: /lib/modules/%{kernel_version}hugemem
Provides: kernel-modules

%description modules-hugemem
This package contains the Global File System (GFS) kernel modules for a hugemem kernel.
%endif

####################################################################################
##  THE PREP SECTION
####################################################################################
%prep
%setup -c -q

%{__cat} <<EOF >%{my_requires}
#!/bin/sh
%{__find_requires} | grep -v '^Perl(Net::SSLeay'
exit 0
EOF
chmod +x %{my_requires}
%define __find_requires %{my_requires}
sed -i "s/v6\.0\.2/%{version}/" `find . -type f`

cd gfs-%{version}/GFS/kernel_patches
rsync -a /usr/src/linux-%{kernel_version} ./
mv linux-%{kernel_version} linux-orig
cd ..
make patches
cd kernel_patches
rm -rf %{kernel_version}
mkdir %{kernel_version}
cp 2.4.21-30.EL/00*patch %{kernel_version}
mv *.patch %{kernel_version}
cd ../../..

cd gfs-%{version}/gnbd/kernel_patches
rsync -a /usr/src/linux-%{kernel_version} ./
mv linux-%{kernel_version} linux-orig
cd ..
make patches
cd kernel_patches
rm -rf %{kernel_version}
mkdir %{kernel_version}
cp 2.4.21-30.EL/00*patch %{kernel_version}
mv *.patch %{kernel_version}
cd ../../..

####################################################################################
##  THE BUILD SECTION
####################################################################################
%build
BUILD_TOPDIR=`pwd`/gfs-%{version}

BuildSistina(){
	cpu_type=$1
	flavor=$2
	kernel_src=/usr/src/linux-%{kernel_version}
	# Kernel tree setup
	if [ -d $kernel_src/. ]; then
		echo "Kernel found."
	else
		echo "Kernel not found."
		ls /lib/modules/2.4.21-*EL
		exit 1
	fi

	cd gfs-%{version}
	rsync -aH $kernel_src/ linux$flavor
	make -C linux$flavor mrproper
	[ -d linux$flavor/configs ] || exit 1
	cat > linux$flavor/.config <<EOF
CONFIG_BLK_DEV_GNBD=m
CONFIG_BLK_DEV_POOL=m
CONFIG_LOCK_HARNESS=m
CONFIG_GFS_FS=m
CONFIG_LOCK_NOLOCK=m
CONFIG_LOCK_GULM=m
EOF
	cat linux$flavor/configs/kernel*${cpu_type}${flavor:+-}$flavor.config >> linux$flavor/.config
	for I in */kernel_patches/%{kernel_version}/*patch; do
		patch -d linux$flavor -p1 -f -i ../$I
	done
	make -C linux$flavor oldconfig \
		VERSION=%{KERNEL_VERSION} \
		PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
		SUBLEVEL=%{KERNEL_SUBLEVEL} \
		EXTRAVERSION=-%{KERNEL_EXTRAVERSION}$flavor

	make -C linux$flavor dep \
		VERSION=%{KERNEL_VERSION} \
		PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
		SUBLEVEL=%{KERNEL_SUBLEVEL} \
		EXTRAVERSION=-%{KERNEL_EXTRAVERSION}$flavor

	# Configure subdirs
	for I in GFS gnbd bedrock; do
		pushd $I
			./configure --mandir=%{_mandir} --kernel_src=$BUILD_TOPDIR/linux$flavor
		popd
	done
	pushd gulm
	./configure --mandir=%{_mandir} --kernel_src=$BUILD_TOPDIR/linux$flavor --libdir=%{_libdir}
	popd
	# Build it!
	make %{?_smp_mflags} ${flavor:-NO_FLAVOR}=y \
		VERSION=%{KERNEL_VERSION} \
		PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
		SUBLEVEL=%{KERNEL_SUBLEVEL} \
		EXTRAVERSION=-%{KERNEL_EXTRAVERSION}$flavor
	cd ..
}

%if %{buildhugemem}
BuildSistina %{_target_cpu} hugemem
%endif

%if %{buildsmp}
BuildSistina %{_target_cpu} smp
%endif

%if %{buildup}
BuildSistina %{_target_cpu}
%endif

####################################################################################
##  THE INSTALL SECTION
####################################################################################
%install
rm -rf $RPM_BUILD_ROOT
cd gfs-%{version}
%if %{buildhugemem}
make install INSTALL_MOD_PATH=$RPM_BUILD_ROOT \
	DESTDIR=$RPM_BUILD_ROOT \
	VERSION=%{KERNEL_VERSION} \
	PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
	SUBLEVEL=%{KERNEL_SUBLEVEL} \
	EXTRAVERSION=-%{KERNEL_EXTRAVERSION}hugemem \
	hugemem=y
%endif

%if %{buildsmp}
make install INSTALL_MOD_PATH=$RPM_BUILD_ROOT \
	DESTDIR=$RPM_BUILD_ROOT \
	VERSION=%{KERNEL_VERSION} \
	PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
	SUBLEVEL=%{KERNEL_SUBLEVEL} \
	EXTRAVERSION=-%{KERNEL_EXTRAVERSION}smp \
	smp=y
%endif

%if %{buildup}
make install INSTALL_MOD_PATH=$RPM_BUILD_ROOT \
	DESTDIR=$RPM_BUILD_ROOT \
	VERSION=%{KERNEL_VERSION} \
	PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
	SUBLEVEL=%{KERNEL_SUBLEVEL} \
	EXTRAVERSION=-%{KERNEL_EXTRAVERSION}
%endif

rm -rf $RPM_BUILD_ROOT/lib/modules/*/build
rm -rf $RPM_BUILD_ROOT/lib/modules/*/pcmcia
cd ..


####################################################################################
##  THE CLEAN SECTION
####################################################################################
%clean
rm -rf $RPM_BUILD_ROOT


####################################################################################
##  THE POST SECTION
####################################################################################
%post
for i in ccsd pool gfs lock_gulmd gnbd_import gnbd_export; do
  /sbin/chkconfig --add $i
done

%post modules
depmod -ae -F /boot/System.map-%{kernel_version} %{kernel_version}

%if %{buildsmp}
%post modules-smp
depmod -ae -F /boot/System.map-%{kernel_version}smp %{kernel_version}smp
%endif

%if %{buildhugemem}
%post modules-hugemem
depmod -ae -F /boot/System.map-%{kernel_version}hugemem %{kernel_version}hugemem
%endif

####################################################################################
##  THE PREUN SECTION
####################################################################################
%preun
if [ "$1" -eq 0 ]; then
for i in ccsd pool gfs lock_gulmd gnbd_import gnbd_export; do
  /sbin/chkconfig --del $i
done
fi

####################################################################################
##  THE POSTUN SECTION
####################################################################################
%postun
if [ -e /dev/poolcomm ] ; then
	rm -f /dev/poolcomm
fi

####################################################################################
##  THE FILES SECTION
####################################################################################
%files
%defattr(-,root,root,-)
# Binaries
/sbin/*
/etc/init.d/*
/%{_libdir}/*.so*

# Config Files
%config(noreplace) /etc/sysconfig/gfs

# Man pages
%{_mandir}/*/*

%if %{buildup}
%files modules
%defattr(-,root,root,-)
# Modules
/lib/modules/%{kernel_version}/kernel/drivers/block/gnbd/gnbd.o
/lib/modules/%{kernel_version}/kernel/drivers/block/gnbd/gnbd_serv.o
/lib/modules/%{kernel_version}/kernel/drivers/md/pool/pool.o
/lib/modules/%{kernel_version}/kernel/fs/gfs/gfs.o
/lib/modules/%{kernel_version}/kernel/fs/gfs_locking/lock_gulm/lock_gulm.o
/lib/modules/%{kernel_version}/kernel/fs/gfs_locking/lock_harness/lock_harness.o
/lib/modules/%{kernel_version}/kernel/fs/gfs_locking/lock_nolock/lock_nolock.o
%endif

%if %{buildsmp}
%files modules-smp
%defattr(-,root,root,-)
# Modules
/lib/modules/%{kernel_version}smp/kernel/drivers/block/gnbd/gnbd.o
/lib/modules/%{kernel_version}smp/kernel/drivers/block/gnbd/gnbd_serv.o
/lib/modules/%{kernel_version}smp/kernel/drivers/md/pool/pool.o
/lib/modules/%{kernel_version}smp/kernel/fs/gfs/gfs.o
/lib/modules/%{kernel_version}smp/kernel/fs/gfs_locking/lock_gulm/lock_gulm.o
/lib/modules/%{kernel_version}smp/kernel/fs/gfs_locking/lock_harness/lock_harness.o
/lib/modules/%{kernel_version}smp/kernel/fs/gfs_locking/lock_nolock/lock_nolock.o
%endif

%if %{buildhugemem}
%files modules-hugemem
%defattr(-,root,root,-)
# Modules
/lib/modules/%{kernel_version}hugemem/kernel/drivers/block/gnbd/gnbd.o
/lib/modules/%{kernel_version}hugemem/kernel/drivers/block/gnbd/gnbd_serv.o
/lib/modules/%{kernel_version}hugemem/kernel/drivers/md/pool/pool.o
/lib/modules/%{kernel_version}hugemem/kernel/fs/gfs/gfs.o
/lib/modules/%{kernel_version}hugemem/kernel/fs/gfs_locking/lock_gulm/lock_gulm.o
/lib/modules/%{kernel_version}hugemem/kernel/fs/gfs_locking/lock_harness/lock_harness.o
/lib/modules/%{kernel_version}hugemem/kernel/fs/gfs_locking/lock_nolock/lock_nolock.o
%endif

%files devel
%defattr(-,root,root,-)
%{_libdir}/libgulm.a
%{_includedir}/libgulm.h

%changelog
* Wed Sep 26 2007 Chris Feist <cfeist@Redhat.com> 6.0.2.36-5
- Rebuilt for 2.4.21-52.EL kernel

* Tue Aug 14 2007 Chris Feist <cfeist@Redhat.com> 6.0.2.36-4
- Rebuilt for 2.4.21-51.EL kernel

* Fri Jun 01 2007 Chris Feist <cfeist@redhat.com> 6.0.2.36-3
- Rebuilt for 2.4.21-47.0.1.EL kernel

* Fri May 18 2007 Chris Feist <cfeist@redhat.com> 6.0.2.36-2
- Rebuilt for new 2.4.21-50.EL kernel

* Thu Nov 09 2006 Chris Feist <cfeist@redhat.com> 6.0.2.36-1
- Rebuilt with new sources to fix assert problems (bz #210403)

* Fri Oct 27 2006 Chris Feist <cfeist@redhat.com> 6.0.2.35-2
- Rebuilt for new 2.4.21-47.0.1.EL kernel

* Mon Sep 18 2006 Chris Feist <cfeist@redhat.com> 6.0.2.35-1
- Rebuilt for new upstream sources.

* Thu Aug 24 2006 Chris Feist <cfeist@redhat.com> 6.0.2.34-3
- Added patch to fix to flock issues (bz #203417).

* Fri Jun 16 2006 Chris Feist <cfeist@redhat.com> 6.0.2.34-0
- Build for new upstream sources & new kernel.

* Mon Apr 28 2006 Chris Feist <cfeist@redhat.com> 6.0.2.31-0
- Build for new upstream sources.

* Mon Mar 6 2006 Chris Feist <cfeist@redhat.com> 6.0.2.30-0
- Build for new upstream sources.

* Thu Dec 8 2005 Chris Feist <cfeist@redhat.com> 6.0.2.28-0
- Build for new upstream sources.

* Wed Sep 7 2005 Chris Feist <cfeist@redhat.com> 6.0.2.27
- Build for new upstream sources.

* Thu May 12 2005 Chris feist <cfeist@redhat.com> 6.0.2.20
- Build for new upstream sources.

* Mon May 02 2005 Chris Feist <cfeist@redhat.com> 6.0.2.17-0
- Build for new kernel 2.4.21-32

* Mon Mar 07 2005 Chris Feist <cfeist@redhat.com> 6.0.2-26
- Added GNBD init.d scripts.

* Fri Jan 14 2005 Chris Feist <cfeist@redhat.com> 6.0.2-25
- Build for new kernel (2.4.21-27.0.2.EL).

* Tue Dec 21 2004 Chris Feist <cfeist@redhat.com> 6.0.2-23
- Build for new kernel (2.4.21-27.0.1.EL).

* Thu Dec 2 2004 Chris Feist <cfeist@redhat.com> 6.0.2-16
- Added /etc/sysconfig/gfs file to rpm. (rbz141628)
- Fixed functions that create variable size arrays on the stack which caused
  a kernel panic. (rbz139863)

* Mon Nov 29 2004 Chris Feist <cfeist@redhat.com> 6.0.2-11
- Build for security release for RHEL3 U3.

* Tue Nov 23 2004 Chris Feist <cfeist@redhat.com> 6.0.2-10
- Fix .spec file to provide 'kernel-modules'.

* Tue Nov 15 2004 Chris Feist <cfeist@redhat.com> 6.0.2-0
- Build for new kernel.
- Fixed circular dependency in GFS fsck (rbz137256).
- gulm_tool prints resources/services registered with lock_gulmd (rbz136220).
- init.d/lock_gulmd will not start if quorum is not established after
  a specified time (rbz135732).
- init.d/lock_gulmd will not stop if GFS is mounted (rbz135730).
- pool init.d scripts no longer hang on startup until console input
  is provided (rbz137382).
- pool_assemble now contains the override option (-O) (rbz137381).
- Support for perl-Crypt-SSLeay added to fence_ilo (rbz137037).
- Support added for configurable SSL port locations in fence_ilo (rbz137035).
- Panics now print out the whole panic string as part of the panic.
  Before, the panic message itself didn't contain any information.
  The useful stuff was printed with printk()s before the panic and
  might not have been seen in a setup without serial consoles.
- A fix for BZ #135684.  Fixed a panic caused by incorrect locking
  of the list of recently used resource groups.  Reported and patched
  by alexander.laamanen@tecnomen.com.

* Tue Nov 9 2004 Chris Feist <cfeist@redhat.com> 6.0.0-17
- Make /etc/sysconfig/gfs a config file in the .spec file.
- Added fence_bladecenter to the file list (rbz136585).

* Thu Nov 4 2004 Chris Feist <cfeist@redhat.com> 6.0.0-16
- Fix __find_requires so it doesn't create a /tmp/my_requires
  (bz #136383)

* Thu Sep 23 2004 AJ Lewis <alewis@redhat.com> 6.0.0-15
- We don't want to error out if there are unpackaged files

* Thu Sep 23 2004 AJ Lewis <alewis@redhat.com> 6.0.0-14
- Pull out Net::SSLeay requirements

* Tue Sep 21 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-13
- fix Makefile typo

* Tue Sep 21 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-12
- Potential fix for (unreproducible) kernel panic thought to be
  caused by misplaced assert statement (rbz133085)
- added new ilo fence agent and man page (rbz132453)

* Thu Aug 19 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-11
- depmod post install has been moved from GFS to GFS-modules* (rbz130773)

* Thu Aug 19 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-10
- fix problem where first 8 chars of node names had to be unique (rbz127828)
- new kernel

* Fri Aug 13 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-9
- Final build for U3

* Mon Aug 2 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-7.1
- Fix stack overflow issue in gulm on x86-64
- Rebuild for kernel 2.4.21-15.0.4.EL (a U2 errata kernel)

* Thu Jul 22 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-8
- Test build for U3

* Thu Jul 8 2004 AJ Lewis <alewis@redhat.com> 6.0.0-7
- Rebuild

* Thu Jul 8 2004 AJ Lewis <alewis@redhat.com> 6.0.0-6
- Actually include the fix that should have been in 6.0.0-5

* Thu Jul 8 2004 AJ Lewis <alewis@redhat.com> 6.0.0-5
- Additional fix for #127031 - Increase initial jid mapping pool size

* Fri Jul 2 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-4
- Fix for #127031 - Add unlock call to jid unlock callback in lockgulm

* Wed Jun 30 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-3
- Add gfs-functions file and update for kernel 2.4.21-15.0.3.EL

* Wed Jun 23 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-2
- Update for new kernel

* Wed Jun 2 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-1.2
- Fix problem with gnbd major number conflict causing kernel panic, final (sbz1333)
- Fix gulm issue that cause GFS to panic with 
  head.lh_flags & GFS_LOG_HEAD_UNMOUNT (sbz1298)
- Add ability for lock_gulmd to reconnect after it has been killed
  and restarted (sbz1265)
- Fix gnbd SMP issue (sbz1322)
- Fix gfs_mount typo (sbz1319)
- Fix gfs_fsck typo (sbz1327)

* Wed Jun 2 2004 Derek Anderson <danderso@redhat.com> 6.0.0-1.1
- Fix problem with gnbd major number conflict causing kernel panic (sbz1333)

* Fri May 28 2004 Derek Anderson <danderso@redhat.com> 6.0.0-1.0
- Fix problem with gulm generation id mismatch (sbz1300)
- Fix problem with gulm master fencing clients after loss of slave (sbz1297)

* Wed May 26 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.15
- Fix gfs_mount.8 typo (sbz1319)
- Fix lock_gulm issue that occurs if all servers die, again (sbz1300)

* Mon May 24 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.14
- Fix problem with preun script running chkconfig --del on upgrade (sbz1326)
- Fix gfs_fsck problem of counting EA blocks incorrectly (sbz1324)
- Fix gulm bug where it thinks a slave is logged in but isn't (sbz1294)
- Fix master gulm server fencing same node multiple times (sbz1313)
- Fix problem with lock_gulmd crashing when master server dies (sbz1288)

* Fri May 21 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.13
- Fix bug in GFS (rather make GFS work around kernel bug) that causes
  kernel panic with "list_empty(&inode->i_dirty_data_buffers)" sbz(1271)
- Fix problem with lock_gulmd hanging the kernel (sbz1316)
- Fix truncated init scripts definitions (sbz1282)

* Thu May 20 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.12
- Update fence agent man pages (sbz1318)
- CCS does not need to check for ip_interfaces in nodes.ccs (sbz1317)
- Fix GETBLKSIZE64 ioctl call in libiddev (sbz1301)
- Fix lock_gulm issue that occurs if all servers die (sbz1300)
- Fix ACL permissions problem (sbz1309)
- Add ACL mount option description to gfs_mount man page (sbz1308)
- Fix mount hang due to lock_gulm deadlock on recovery (sbz1258)

* Tue May 18 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.11
- Remove hardcoded patch directory to prevent out of sync builds
- gfs_fsck fix for extended attributes (sbz1273)
- Give pool the ability to do read-only devices (sbz1304)

* Mon May 10 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.10
- Rebuild for Pool grow bug fix (sbz1296)
- Increase the number of possible paths for Pool multipathing (sbz1302)
- Fix bug in fence were not all arguments were being passed to fence agent (sbz1299)

* Fri May 7 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.9
- build again, this time with i686 support

* Wed May 5 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.8
- Forgot to include the fence_mcdata man page

* Tue May 4 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.7
- New build for new collection instance

* Tue Apr 20 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.6
- Pull gfs_eattr and man page.  There are now native ways of doing this.
- Add init scripts for gfs, pool, ccsd, and lock_gulmd

* Mon Apr 19 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.5
- Add fence_xcat and its man page to the files list
- Turn on smp package build

* Fri Apr 16 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.5
- Move items around in the files section so everything is in the
  correct package
- Exclude certain archs which aren't supported
- Specify kernel version info to all make targets so that modules install
  to the correct spot
- Add --libdir arg to configure so gulm can put it's libs in the right spot

* Tue Apr 13 2004 Elliot Lee <sopwith@redhat.com> 6.0.0-0.1
- Avoid using /tmp/my_requires (security hole), and avoid filtering
  requires altogether, since the %pre scripts are trying to do the
  equivalent of dependencies anyways
- Remove Provides: kernel-modules (since this is probably not what is intended).
- On a wild guess, change Conflicts: to Obsoletes:. The theory here is
  that the functionality formerly in GFS-gnbd and GNBD is now
  contained in this package - for that situation, Obsoletes: is more
  appropriate.
- Rework the subpackages so that the userland utils are all in the main
  package, and the kernel modules are in subpackages (one per kernel
  variant).
- Use _smp_mflags to speed up the build on SMP machines.
- Eliminate perl as a build-time dependency by doing the equivalent
  of ./configure inside the .spec file in fewer lines.
- BuildRequires on kernel/kernel-smp/kernel-hugemem (must be unconditional)
